/***************************************************************************
 *   Copyright (C) 2009 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qpixmap.h>
#include <qpainter.h>
#include <qbuffer.h>
#include <qimage.h>
#include <kdebug.h>
#include "kxstitchdragobject.h"
#include "stitch.h"
#include "configuration.h"

KXStitchDragObject::KXStitchDragObject(QByteArray data, QWidget *source, char *name)
  : QDragObject(source, name),
    m_data(data)
{
  m_outputFormats.append("application/kxstitch");
  QStrList ofmts = QImageIO::outputFormats();
  for (int i = 0 ; i < ofmts.count() ; i++)
  {
    QString f = QString("image/%1").arg(QString(ofmts.at(i)).lower());
    m_outputFormats.append(f.latin1());
  }
  setScale(Configuration::library_IconViewScale());
}

const char *KXStitchDragObject::format(int n) const
{
  if (m_outputFormats.count() < n)
    return 0;
  QStrListIterator it(m_outputFormats);
  while (n--)
    ++it;
  return it.current();
}

bool KXStitchDragObject::provides(const char *mimeType) const
{
  int i;
  QStrListIterator it(m_outputFormats);
  for (i = 0 ; i < m_outputFormats.count() ; i++, ++it)
    if (qstrcmp(it.current(),mimeType) == 0) break;
  return i < m_outputFormats.count();
}

QByteArray KXStitchDragObject::encodedData(const char *mimeType) const
{
  QPixmap       pixmap;
  QByteArray    data;
  QBuffer       buffer;
  QImageIO      io;
  QDataStream   read(m_data, IO_ReadOnly);
  QPainter      painter;
  QString       scheme;
  Q_INT32       width;
  Q_INT32       height;
  Q_INT8        count;
  Q_INT8        type;
  Stitch::Type  stitchType;
  Q_INT32       backstitches;
  Q_INT32       knots;
  QPoint        start;
  QPoint        end;
  QColor        color;
  if (qstrcmp(mimeType, "application/kxstitch") == 0)
  {
    data = m_data;
  }
  else
  {
    if (provides(mimeType)) // application/kxstitch has already been checked for so this will be an image format
    {
      read >> scheme >> width >> height;
      pixmap.resize(width*m_scale+1, height*m_scale+1);
      pixmap.fill();
      painter.begin(&pixmap);
      painter.setPen(QPen(Qt::lightGray));

      for (int dx = 0 ; dx <= width ; dx++)
      {
        // draw vertical grid line
        painter.drawLine(dx*m_scale, 0, dx*m_scale, height*m_scale);
      }

      for (int dy = 0 ; dy <= height ; dy++)
      {
        // draw horizontal grid line
        painter.drawLine(0, dy*m_scale, width*m_scale, dy*m_scale);
      }

      for (int dy = 0 ; dy < height ; dy++)
      {
        for (int dx = 0 ; dx < width ; dx++)
        {
          QRect r(dx*m_scale, dy*m_scale, m_scale, m_scale);
          read >> count;
          while (count--)
          {
            read >> type >> color;
            stitchType = (Stitch::Type)type;
            painter.setPen(color);
            if (stitchType & Stitch::TLQtr)
              painter.drawLine(r.topLeft(), r.center());
            if (stitchType & Stitch::TRQtr)
              painter.drawLine(r.topRight(), r.center());
            if (stitchType & Stitch::BLQtr)
              painter.drawLine(r.bottomLeft(), r.center());
            if (stitchType & Stitch::BRQtr)
              painter.drawLine(r.bottomRight(), r.center());
          }
        }
      }

      read >> backstitches;
      while (backstitches--)
      {
        read >> start >> end >> color;
        painter.setPen(color);
        painter.drawLine(start*m_scale/2, end*m_scale/2);
      }

      read >> knots;
      while (knots--)
      {
        read >> start >> color;
        painter.setPen(color);
        painter.setBrush(color);
        painter.drawEllipse(QRect(start*m_scale/2-QPoint(1, 1), start*m_scale/2+QPoint(2, 2)));
      }

      painter.end();
      buffer.setBuffer(data);
      buffer.open(IO_WriteOnly);
      QString type = QString(mimeType);
      type = (type.remove(0, type.find('/')+1)).upper();
      io.setIODevice(&buffer);
      io.setFormat(type);
      io.setImage(pixmap.convertToImage());
      io.write();
      buffer.close();
    }
  }
  return data;
}

void KXStitchDragObject::setScale(int scale)
{
  m_scale = scale;
  setPixmap(QPixmap(encodedData("image/png")), QPoint(scale/2, scale/2));
}
