/***************************************************************************
 *   Copyright (C) 2003 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef __LAYOUTTEMPLATE_DIALOG_H
#define __LAYOUTTEMPLATE_DIALOG_H

#include "layouttemplatedlg.h"
#include <qptrlist.h>
#include <qstringlist.h>
#include <kprinter.h>

class QPainter;

/** Wrapper class to manage the layout template dialog
  */

class LayoutTemplatePageElement : public QWidget
{
  Q_OBJECT
public:
  LayoutTemplatePageElement(QWidget *);
  ~LayoutTemplatePageElement();
  virtual void print(QPainter *);
};

class LayoutTemplatePage : public QWidget
{
  Q_OBJECT
public:
  LayoutTemplatePage(QWidget *);
  ~LayoutTemplatePage();

  void setOrientation(KPrinter::Orientation);
  enum KPrinter::Orientation  orientation() const;
  void setPageSize(KPrinter::PageSize);
  void setPageSize(int);
  enum KPrinter::PageSize pageSize() const;
  int width() const;
  int height() const;
  void setPageNumber(int);
  int pageNumber() const;
  void print(KPrinter *);

protected:
  virtual void paintEvent(QPaintEvent *);

protected slots:
  void mousePressEvent(QMouseEvent *);
  void mouseMoveEvent(QMouseEvent *);
  void mouseReleaseEvent(QMouseEvent *);
  void contextMenuEvent(QContextMenuEvent *);
  void slotElementPageNumber();
  void slotElementTitle();
  void slotElementFabric();
  void slotElementFabricColor();
  void slotElementPageGrid();
  void slotElementCopyright();
  void slotElementAuthor();
  void slotPagePortrait();
  void slotPageLandscape();
  void slotPageA0();
  void slotPageA1();
  void slotPageA2();
  void slotPageA3();
  void slotPageA4();
  void slotPageA5();
  void slotPageA6();
  void slotPageA7();
  void slotPageA8();
  void slotPageA9();
  void slotPageB0();
  void slotPageB1();
  void slotPageB2();
  void slotPageB3();
  void slotPageB4();
  void slotPageB5();
  void slotPageB6();
  void slotPageB7();
  void slotPageB8();
  void slotPageB9();
  void slotPageB10();
  void slotPageComm10E();
  void slotPageDLE();
  void slotPageExecutive();
  void slotPageFolio();
  void slotPageLedger();
  void slotPageLegal();
  void slotPageLetter();
  void slotPageTabloid();

private:
  void updatePageDisplay();
  void drawGrid(QPainter *);

  enum KPrinter::PageSize m_pageSize;
  enum KPrinter::Orientation m_orientation;
  enum KPrinter::ColorMode m_colorMode;
  int m_width;
  int m_height;
  int m_marginLeft;
  int m_marginRight;
  int m_marginTop;
  int m_marginBottom;
  int m_pageNumber;
  int m_gridX;
  int m_gridY;
  int m_gridXStart;
  int m_gridYStart;
  bool m_showGrid;
  QPtrList<LayoutTemplatePageElement> m_pageElements;
};

class LayoutTemplateDialog : public LayoutTemplateDlg
{
  Q_OBJECT
public:
  LayoutTemplateDialog(QWidget *);
  virtual ~LayoutTemplateDialog();

protected slots:
  virtual void accept();
  void slotActivatedPageSize(int);
  void slotNextPage();
  void slotPrevPage();
  void slotRotatePage();
  void slotInsertPage();
  void slotAppendPage();
  void slotDeletePage();
  void slotSetPageSize(int);
  void slotContextHelp();
  void slotZoomIn();
  void slotZoomOut();

private:
  void updatePageNumbers();
  void updatePageOfPages();
  void updateButtons();

  int m_currentPage;
  QPtrList<LayoutTemplatePage> m_pages;
};

#endif // __LAYOUTTEMPLATE_DIALOG_H
