/***************************************************************************
 *   Copyright (C) 2003 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kdebug.h>
#include <kiconview.h>
#include "librarypattern.h"
#include "keycodelineedit.h"

LibraryPattern::LibraryPattern(Q_INT32 key, Qt::ButtonState modifiers, Q_INT16 baseline, Q_UINT16 checksum, QByteArray &data)
  : m_key(key),
    m_modifiers(modifiers),
    m_baseline(baseline),
    m_checksum(checksum),
    m_data(data),
    m_iconViewItem(0),
    m_changed(false)
{
  QDataStream stream(data, IO_ReadOnly);
  stream >> m_scheme >> m_width >> m_height;
}

Q_INT32 LibraryPattern::key()
{
  return m_key;
}

Qt::ButtonState LibraryPattern::modifiers()
{
  return m_modifiers;
}

void LibraryPattern::setKeyModifiers(Q_INT32 key, Qt::ButtonState modifiers)
{
  m_key = key;
  m_modifiers = modifiers;
  m_iconViewItem->setText(KeycodeLineEdit::keyString(m_key, m_modifiers));
  m_changed = true;
}

Q_INT16 LibraryPattern::baseline()
{
  return m_baseline;
}

void LibraryPattern::setBaseline(Q_INT16 baseline)
{
  m_baseline = baseline;
  m_changed = false;
}

Q_UINT16 LibraryPattern::checksum()
{
  return m_checksum;
}

QByteArray LibraryPattern::data()
{
  return m_data;
}

QString LibraryPattern::scheme()
{
  return m_scheme;
}

unsigned LibraryPattern::width()
{
  return m_width;
}

unsigned LibraryPattern::height()
{
  return m_height;
}

KIconViewItem *LibraryPattern::iconViewItem()
{
  return m_iconViewItem;
}

void LibraryPattern::setIconViewItem(KIconViewItem *item)
{
  m_iconViewItem = item;
  item->setText(KeycodeLineEdit::keyString(m_key, m_modifiers));
}

bool LibraryPattern::hasChanged()
{
  return m_changed;
}
